<?php
/*
Plugin Name: Adhesive
Plugin URI: http://www.asymptomatic.net/wp-hacks
Description: Allows easy marking of sticky posts in WordPress 1.3.<br/>Licensed under the <a href="http://www.opensource.org/licenses/mit-license.php">MIT License</a>, Copyright 2004 Owen Winkler.
Version: 2.1 beta
Author: Owen Winkler
Author URI: http://www.asymptomatic.net
*/

/*
Adhesive - Allows easy marking of sticky posts in WordPress.
Copyright (c) 2004 Owen Winkler

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the
Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to
do so, subject to the following conditions:

The above copyright notice and this permission notice shall
be included in all copies or substantial portions of the
Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

/*

INSTRUCTIONS:

No edits to the WordPress source are required.

Just drop this into your WordPress plugins directory, and then
Activate it on the Plugins tab of the WordPress admin console.

A new checkbox will appear in the Post Status area labeled "Sticky".
Check the Sticky box to make a post sticky (appear at the top)
within any category to which it belongs.

A sticky post is assigned the additional CSS class "adhesive_post"
using script that is inserted into its content by the plugin.
To change the look of a sticky post, add CSS to your wp-layout.css
file.  For example:

.adhesive_post {
	border: 1px solid green;
	margin: 15px;
}
.adhesive_post H3 {
	background-color: yellow;
}
.adhesive_post .storycontent p {
	font-weight: bold;
}

If you want your sticky posts to display a date, you should
delete the line below that reads:
add_filter('the_date', 'adhesive_the_date', 15);




For Developers:
This plugin relies on the $posts array being mostly tamper-free.
If you are using other plugins that interfere with the $posts
array, you may have difficulty getting this plugin to work.

*/


// $sticky_date_text is the text that appears instead of a date above the sticky post.
//     Set $sticky_date_text = '' to display nothing instead of a date.
$sticky_date_text = '<h1 title="Change this heading in the WordPress adhesive.php plugin!">Important Message!</h1>';

// $only_on_category_pages
//     == true -> Sticky posts stick only to category pages where they are filed.
//     == false -> Sticky posts appear on the home page, too.
$only_on_category_pages = false;


// Delete the following line to display the date on sticky posts
add_filter('the_date', 'adhesive_the_date', 15);




// Don't change below here.

add_filter('the_posts', 'adhesive_the_posts');
add_filter('posts_where', 'adhesive_posts_where');
add_filter('admin_footer', 'adhesive_admin_footer');
add_filter('edit_post', 'adhesive_edit_post');
add_filter('publish_post', 'adhesive_edit_post');
add_filter('the_content', 'adhesive_the_content', 15);
add_filter('the_title', 'adhesive_the_title');

$adhesive_metakey = 'sticky';


function adhesive_posts_where($where)
{
	global $only_on_category_pages;
	if(!$only_on_category_pages || is_category())
		$where = ' AND   0=1   ' . $where;
	return $where;
}

function adhesive_the_posts($posts)
{
	global $wpdb, $adhesive_metakey, $request, $q, $only_on_category_pages;

	if(!$only_on_category_pages || is_category())
	{
		$qry = "SELECT {$wpdb->postmeta}.Post_ID FROM {$wpdb->postmeta} WHERE meta_key = '{$adhesive_metakey}' AND meta_value = '1'";
		if($stickies = $wpdb->get_col($qry))
		{
			$insticky = 'not (ID in (' . implode(',', $stickies) . ')),';
		}
		else
		{
			$insticky = '';
			$stickies = array();
		}

		$request = preg_replace("/ORDER BY post_{$q['orderby']}/", "ORDER BY {$insticky} post_{$q['orderby']}", $request);
		$request = str_replace(' AND   0=1   ', '', $request);

		unset($posts);
		$posts = array();
		if($foo = $wpdb->get_results($request))
		{
			foreach($foo as $post)
			{
				$post->is_sticky = (in_array($post->ID, $stickies));
				array_push($posts, $post);
			}
		}
	}

	return $posts;
}

function adhesive_admin_footer($content)
{
	global $wpdb, $exc, $adhesive_metakey;

	// Are we on the right page?
	if(preg_match('|post.php|i', $_SERVER['SCRIPT_NAME']) && $_REQUEST['action'] == 'edit')
	{
		if(isset($_REQUEST['post']))
		{
			$stickiness = $wpdb->get_var("SELECT meta_value FROM {$wpdb->postmeta} WHERE Post_ID = {$_REQUEST['post']} AND meta_key = '{$adhesive_metakey}';");
			$checked = ('1' == $stickiness)?' checked="checked"':'';
		}
?>
<div id="adhesivediv"><label class="selectit"><input type="checkbox" name="adhesive_sticky" value="true"<?php echo $checked; ?> /> Sticky</label></div>
<?php

		echo '
		<script language="JavaScript" type="text/javascript"><!--
		var placement = document.getElementById("post_status_private");
		var substitution = document.getElementById("adhesivediv");
		var mozilla = document.getElementById&&!document.all;
		if(mozilla)
			placement.parentNode.parentNode.appendChild(substitution);
		else placement.parentElement.parentElement.appendChild(substitution);

		//--></script>
		';

	}
}

function adhesive_edit_post($id)
{
	global $wpdb, $adhesive_metakey;

	if(!isset($id)) $id= $_REQUEST['post_ID'];

	if(isset($_REQUEST['adhesive_sticky']))
	{
		$qry = "DELETE FROM {$wpdb->postmeta} WHERE Post_ID = {$id} AND meta_key = '{$adhesive_metakey}';";
		$wpdb->query($qry);
		$qry = "INSERT INTO {$wpdb->postmeta} (Post_ID, meta_key, meta_value) VALUES ({$id}, '{$adhesive_metakey}', '1');";
		$wpdb->query($qry);
	}
	else
	{
		$qry = "DELETE FROM {$wpdb->postmeta} WHERE Post_ID = {$id} AND meta_key = '{$adhesive_metakey}';";
		$wpdb->query($qry);
	}
}

function adhesive_the_content($content)
{
	global $post;
	return ($post->is_sticky) ?
		"<script type=\"text/javascript\">window.document.getElementById('post-{$post->ID}').parentNode.className += ' adhesive_post';</script>{$content}" :
		$content;
}

function adhesive_the_date($content)
{
	global $post, $previousday, $sticky_date_text;
	if($post->is_sticky)
	{
		$previousday='';
		return $sticky_date_text;
	}
	return $content;
}

function adhesive_the_title($content)
{
	global $post;
	return (preg_match('|edit.php|i', $_SERVER['SCRIPT_NAME']) && ($post->is_sticky)) ?
		"<strong>Sticky:</strong>{$content}" :
		$content;
}

?>